#include <proto.h>

#include "gtpalette.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtPalette class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init palette.
*
\************************************************************************/
GtPalette *GtMkPalette(char *n,GtManager *p,GtTag *tgs)
{
	GtPalette *obj;
	
	if(!(obj=new GtPalette()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtPalette object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init palette.
*
\************************************************************************/
GtPalette *GtMkPalette(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtPalette *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkPalette(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty palette.
*
\************************************************************************/
GtPalette::GtPalette(void)
{
	cmap=NULL;
	nrcol=0;
	cols=0;
	rows=0;
	mode=GTPM_SELECT;
	bdrw=1;
	sel=FALSE;
}

/************************************************************************\
*
* Setup palette.
*
\************************************************************************/
sbool GtPalette::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_BDR,GTRT_BYTE,&bdrw);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Mode:
				mode=t->d.val;
				if(mode>GTPM_LAST_NR)
					mode=GTPM_SELECT;
				break;
			case GT_ColorArray:
				cmap=(ulong *)t->d.ptr;
				break;
			case GT_Colors:
				nrcol=t->d.val;
				break;
			case GT_Columns:
				cols=t->d.val;
				break;
			case GT_Rows:
				rows=t->d.val;
				break;
			case GT_Select:
				selnr=t->d.val;
				break;
			case GT_Border:
				bdrw=t->d.val;
				break;
		}
	}
	this->PrivateGC(GT_Foreground);
	if(!cmap)
	{
		GtError(this,GTEC_MISC,"No color array!\n");
		return(FALSE);
	}
	if(!cols && !rows)
	{
		while(cols*cols<nrcol)
			cols++;
		rows=(nrcol+cols-1)/cols;
	}
	else if(!rows)
	{
		rows=(nrcol+cols-1)/cols;
	}
	else if(!cols)
	{
		cols=(nrcol+rows-1)/rows;
	}
	this->SetMinMaxSize((2*bdrw+5)*cols,(2*bdrw+5)*rows,GTS_MAX,GTS_MAX);
	return(TRUE);
}

/************************************************************************\
*
* Set palette attributes.
*
\************************************************************************/
sbool GtPalette::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE,sch=FALSE;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Mode:
				mode=t->d.val;
				if(mode>GTPM_LAST_NR)
					mode=GTPM_SELECT;
				ch=TRUE;
				break;
			case GT_ColorArray:
				cmap=(ulong *)t->d.ptr;
				ch=TRUE;
				break;
			case GT_Colors:
				nrcol=t->d.val;
				sch=TRUE;
				break;
			case GT_Columns:
				cols=t->d.val;
				sch=TRUE;
				break;
			case GT_Rows:
				rows=t->d.val;
				sch=TRUE;
				break;
			case GT_Select:
				selnr=t->d.val;
				ch=TRUE;
				break;
			case GT_Border:
				bdrw=t->d.val;
				ch=TRUE;
				break;
		}
	}
	if(ch && !cmap)
	{
		GtError(this,GTEC_MISC,"No color array!\n");
		return(FALSE);
	}
	if(sch)
	{
		if(!cols && !rows)
		{
			while(cols*cols<nrcol)
				cols++;
			rows=(nrcol+cols-1)/cols;
		}
		else if(!rows)
		{
			rows=(nrcol+cols-1)/cols;
		}
		else if(!cols)
		{
			cols=(nrcol+rows-1)/rows;
		}
		this->SetMinMaxSize((2*bdrw+5)*cols,(2*bdrw+5)*rows,GTS_MAX,GTS_MAX);
	}
	if(ch || sch)
		this->Refresh();
	return(TRUE);
}

/************************************************************************\
*
* Get palette attributes.
*
\************************************************************************/
void GtPalette::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Mode:
				*ptr=mode;
				break;
			case GT_ColorArray:
				*ptr=(long)cmap;
				break;
			case GT_Colors:
				*ptr=nrcol;
				break;
			case GT_Columns:
				*ptr=cols;
				break;
			case GT_Rows:
				*ptr=rows;
				break;
			case GT_Select:
				*ptr=selnr;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtPalette::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtPalette")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* Set palette event mask.
*
\************************************************************************/
void GtPalette::SetEMask(ulong em)
{
	GtWidget::SetEMask(em|ExposureMask|ButtonPressMask);
}

/************************************************************************\
*
* Add palette callback.
*
\************************************************************************/
void GtPalette::AddCB(ulong tag,GtCallB cb,GtUDPtr ud)
{
	if(tag==GT_Select)
		selcb.AddCB(cb,ud);
	else
		GtWidget::AddCB(tag,cb,ud);
}

/************************************************************************\
*
* Remove palette callback.
*
\************************************************************************/
void GtPalette::RemCB(ulong tag,GtCallB cb)
{
	if(tag==GT_Select)
		selcb.RemCB(cb);
	else
		GtWidget::RemCB(tag,cb);
}

/************************************************************************\
*
* Draw palette.
*
\************************************************************************/
sbool GtPalette::Draw(XEvent *xe)
{
	int xs,ys,xo,yo;
	
	if(xe && (xe->type!=Expose))
		return(TRUE);
	if(nrcol==0)
		return(TRUE);
	xs=xsize/cols;
	ys=ysize/rows;
	xo=(xsize-xs*cols)/2;
	yo=(ysize-ys*rows)/2;
	if(!xe)
		this->ReDraw(0,0,cols,rows);
	else
	{
		this->ReDraw((xe->xexpose.x-xo)/xs,(xe->xexpose.y-yo)/ys,
			1+(xe->xexpose.width+xs-1-xo)/xs,1+(xe->xexpose.height+ys-1-yo)/ys);
	}
	return(TRUE);
}

/************************************************************************\
*
* Mouse events for palette.
*
\************************************************************************/
sbool GtPalette::Mouse(XEvent *xe)
{
	GtPaletteCD cd;
	int xp,yp,xs,ys,xo,yo;
	long tmp;
	
	if(!(flags & GTWF_DISABLED) && (mode==GTPM_SELECT) && 
		(xe->type==ButtonPress) && (xe->xbutton.button==Button1))
	{
		xs=xsize/cols;
		ys=ysize/rows;
		xo=(xsize-xs*cols)/2;
		yo=(ysize-ys*rows)/2;
		xp=(xe->xbutton.x-xo)/xs;
		yp=(xe->xbutton.y-yo)/ys;
		if((xp<cols) && (yp<rows) && (yp*cols+xp<nrcol))
		{
			tmp=selnr;
			selnr=yp*cols+xp;
			this->ReDraw(tmp%cols,tmp/cols,1,1);
			this->ReDraw(xp,yp,1,1);
			cd.color=selnr;
			cd.pixel=cmap[selnr];
			selcb.CallCB(this,&cd);
		}
		return(TRUE);
	}
	return(FALSE);
}

/************************************************************************\
*
* Redraw part of palette.
*
\************************************************************************/
void GtPalette::ReDraw(int xp,int yp,int xs,int ys)
{
	int n,b,x,y,xsz,ysz,c,xo,yo;
	
	if(nrcol==0)
		return;
	if((xp>=cols) || (yp>=rows))
		return;
	if(xp+xs>cols)
		xs=cols-xp;
	if(yp+ys>rows)
		ys=rows-yp;
	xsz=xsize/cols;
	ysz=ysize/rows;
	xo=(xsize-xsz*cols)/2;
	yo=(ysize-ysz*rows)/2;
	x=xsz*xp+xo;
	y=ysz*yp+yo;
	c=cols*yp+xp;
	for(n=0; n<xs*ys; n++)
	{
		if(c>=nrcol)
			break;
		gtfore->SetForeground(cmap[c]);
		gtfore->DrawRectangle(gtwin,x,y,xsz,ysz,TRUE);
		if((selnr==c) && (mode==GTPM_SELECT))
		{
			for(b=0; b<bdrw; b++)
			{
				GT3DBOX(gtdrk,gtlht,gtwin,x+b,y+b,xsz-2*b,ysz-2*b);
			}
		}
		else
		{
			for(b=0; b<bdrw; b++)
			{
				GT3DBOX(gtlht,gtdrk,gtwin,x+b,y+b,xsz-2*b,ysz-2*b);
			}
		}
		c++;
		x+=xsz;
		if(x+xsz>(xp+xs)*xsz+xo)
		{
			x=xsz*xp+xo;
			y+=ysz;
			c+=cols-xs;
		}
	}
}
